<?php

return [

    'authenticate' => [
        'path' => '/api/v1/account/authenticate',
        'method' => 'POST',
        'verify' => true,
    ],

    'activate' => [
        'path' => '/api/v1/account/%s/activate',
        'method' => 'PATCH',
        'verify' => true
    ],

    'create' => [
        'path' => '/api/v1/account',
        'method' => 'POST',
        'verify' => true
    ],

    'read' => [
        'path' => '/api/v1/account/%s',
        'method' => 'GET',
        'verify' => true,
    ],

    'update' => [
        'path' => '/api/v1/account/%s',
        'method' => 'PUT',
        'verify' => true,
    ],

    'delete' => [
        'path' => '/api/v1/account/%s',
        'method' => 'DELETE',
        'verify' => true
    ],

    // Get organization that the account is a member of
    'organizations' => [
        'path' => '/api/v1/account/%s/organizations',
        'method' => 'GET',
        'verify' => true
    ],

    // Join an organization
    'join_organization' => [
        'path' => '/api/v1/account/%s/associate/organization',
        'method' => 'POST',
        'verify' => true
    ],

    // Leave an organization
    'leave_organization' => [
        'path' => '/api/v1/account/%s/leave-organization/%s',
        'method' => 'POST',
        'verify' => true
    ]

];