<?php

return [
    'baseUrl' => isset($baseUrl) && !empty($baseUrl) ? $baseUrl : 'https://localhost',

    'options' => [
        'headers' => [

            'Connection' => 'close',

            'Accept'     => 'application/json',

            'User-Agent' => 'AUD-REST-CONNECTOR/2.0',

            'Authorization' => sprintf("Bearer %s", isset($token) ? $token : '')
        ]
    ],

    'routes' => [

        // Account routes
        'account'       => require_once('account.php'),

        // Organization routes
        'organization'  => require_once('organization.php'),

        // Policy routes
        'policy'        => require_once('policy.php'),

        // Team routes
        'team'          => require_once('team.php'),

        // Permission routes
        'permission'    => require_once('permission.php'),

        // Role route
        'role'          => require_once('role.php')
    ]
];